@@ -6,6 +6,7 @@ from azol.clients.oauth_http_client import OAuthHTTPClient
 from azol.clients.datafactory_client import DataFactoryClient
 from azol.clients.kudu_client import KuduClient
 from azol.clients.devops_agent_client import AzureDevOpsAgentClient
+from azol.clients.azure_devops_client import AzureDevOpsClient
 
 __all__ = [
     "ArmClient",
@@ -14,5 +15,6 @@ __all__ = [
     "OAuthHTTPClient",
     "DataFactoryClient",
     "KuduClient",
-    "AzureDevOpsAgentClient"
+    "AzureDevOpsAgentClient",
+    "AzureDevOpsClient"
 ]
