@@ -5,11 +5,13 @@ from azol.credentials.service_principal import ServicePrincipal
 from azol.credentials.application_object import ApplicationObject
 from azol.credentials.user import User
 from azol.credentials.azure_devops_agent_key import DevOpsAgentCredential
+from azol.credentials.azure_devops_pipeline_workload_federation_credential import ADOWorkloadFederationCredential
 
 __all__ = [
     "AccessToken",
     "ServicePrincipal",
     "ApplicationObject",
     "User",
-    "DevOpsAgentCredential"
+    "DevOpsAgentCredential",
+    "ADOWorkloadFederationCredential",
 ]
