@@ -2,5 +2,6 @@
 from .windows_utils import *
 
 __all__ = [
-    "decrypt_dpapi"
+    "decrypt_dpapi",
+    "get_azure_cli_credential_file_contents"
 ]
